/*****************************************************************************
 * Copyright (c) 2020, rrcSmall FM10K-Documentation Contributors
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright notice,
 *       this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the copyright holder nor the names of its contributors
 *       may be used to endorse or promote products derived from this software
 *       without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *****************************************************************************/

#include <iostream>
#include <fstream>
#include <sstream>
#include <iomanip>
#include <cstring>

#include "ImageFormat.h"

void decodeImage(const std::string& fileName){

    std::ifstream image(fileName);
    if(image.is_open()){
        std::vector<uint8_t> bytes;
        while(!image.eof()){
            bytes.push_back(image.get());
        }
        auto imageObject = ImageFormat::fromBytes(bytes);

        for(auto& k : imageObject.getBootConfig().getAllEntries()){
            std::cout << k << "\n";
        }

        uint32_t prevAddress = 0;
        for(const auto& entry : imageObject.getInstructions()){
            const auto& instruction = entry.second;
            if(instruction->getAddress() < prevAddress){
                std::cout << "====== DECODE ERROR? ========== " << std::hex << std::setw(8) << std::setfill('0') << prevAddress << " - " << std::hex << std::setw(8) << std::setfill('0') << instruction->getAddress() << "\n";
            }else if((instruction->getAddress() - prevAddress) >= 4 && (instruction->getAddress() - prevAddress) < 1024){
                for(uint32_t addr = prevAddress; addr < instruction->getAddress(); addr += 4){
                    std::stringstream op;
                    std::string printable;
                    op << std::hex << std::setw(8) << std::setfill('0') << addr << ": ";
                    for(uint32_t i = 0; i < 4; ++i){
                        op << std::hex << std::setw(2) << std::setfill('0') << (uint32_t)bytes[addr + i];
                        if(bytes[addr + i] >= 0x20 && bytes[addr + i] < 0x7F){
                            printable += bytes[addr + i];
                        }else{
                            printable += ".";
                        }
                    }
                    op << " " << printable << "\n";

                    std::cout << op.str();
                }
            }else if((instruction->getAddress() - prevAddress) >= 1024){
                std::cout << "================ " << std::hex << std::setw(8) << std::setfill('0') << prevAddress << " - " << std::hex << std::setw(8) << std::setfill('0') << instruction->getAddress() << "\n";
            }
            std::cout << std::hex << std::setw(8) << std::setfill('0') << instruction->getAddress() << ": ";
            std::cout << instruction->toString() << "\n";
            prevAddress = instruction->getEndAddress();
        }
    }
}

void patchImage(const std::string& originalImage, const std::string& settings, const std::string& patchedImageFile){
    std::ifstream image(originalImage);
    if(image.is_open()) {
        std::vector<uint8_t> bytes;
        while (!image.eof()) {
            bytes.push_back(image.get());
        }
        auto imageObject = ImageFormat::fromBytes(bytes);

        //Do patch

        std::ofstream patchedImage(patchedImageFile);
        auto result = imageObject.toBytes();
        for (auto c : result) {
            patchedImage.put(c);
        }
        patchedImage.close();
        image.close();
    }
}

int main(int argc, char* argv[]) {

    if(argc >= 2 && strcmp(argv[1], "decode") == 0){
        decodeImage(argv[2]);
        return 0;
    }else if(argc >= 4 && strcmp(argv[1], "encode") == 0){
        patchImage(argv[2], argv[3], argv[4]);
        return 0;
    }else{
        std::cout << "Usage:\n";
        std::cout << "    decode: " << argv[0] << " decode nvmImage.bin > decoded.txt\n";
        std::cout << "    encode: " << argv[0] << " encode originalNvmImage.bin settings.cfg patchedNvmImage.bin\n";
        return 1;
    }
}
